from __future__ import annotations

from metasim.utils import configclass

from .base_robot_metacfg import BaseActuatorMetaCfg, BaseRobotMetaCfg


@configclass
class FrankaSlideUrdfMetaCfg(BaseRobotMetaCfg):
    name: str = "franka"
    num_joints: int = 9
    urdf_path: str = "assets/franka_description/robots/franka_panda_slider_longer.urdf"
    enabled_gravity: bool = False
    enabled_self_collisions: bool = False
    actuators: dict[str, BaseActuatorMetaCfg] = {
        "z_free_base1": BaseActuatorMetaCfg(velocity_limit=5.0),
        "z_free_base2": BaseActuatorMetaCfg(velocity_limit=5.0),
        "panda_joint1": BaseActuatorMetaCfg(velocity_limit=2.175),
        "panda_joint2": BaseActuatorMetaCfg(velocity_limit=2.175),
        "panda_joint3": BaseActuatorMetaCfg(velocity_limit=2.175),
        "panda_joint4": BaseActuatorMetaCfg(velocity_limit=2.175),
        "panda_joint5": BaseActuatorMetaCfg(velocity_limit=2.61),
        "panda_joint6": BaseActuatorMetaCfg(velocity_limit=2.61),
        "panda_joint7": BaseActuatorMetaCfg(velocity_limit=2.61),
        "panda_finger_joint1": BaseActuatorMetaCfg(velocity_limit=0.2),
        "panda_finger_joint2": BaseActuatorMetaCfg(velocity_limit=0.2),
    }
    ee_prim_path: str = "panda_hand"



## TODO:
## 1. This task is hard because at the beginning, the gripper may hold the block in the air
## 2. The checker is not exactly same as the source simulator
##    In source simulator, we need to check specifically the block that gripper contacts with at the beginning
##    In this implementation, we check if any one block is stacked on any other block

from itertools import product

from metasim.cfg.checkers import DetectedChecker, OrOp, RelativeBboxDetector
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, TaskType
from metasim.utils import configclass

stack_on_red_detector = RelativeBboxDetector(
    base_obj_name="block_red",
    relative_pos=(0.0, 0.0, 0.05),
    relative_quat=(1.0, 0.0, 0.0, 0.0),
    checker_lower=(-0.025, -0.025, -0.025),
    checker_upper=(0.025, 0.025, 0.025),
    ignore_base_ori=True,
    # debug_vis=True,
)

stack_on_blue_detector = RelativeBboxDetector(
    base_obj_name="block_blue",
    relative_pos=(0.0, 0.0, 0.05),
    relative_quat=(1.0, 0.0, 0.0, 0.0),
    checker_lower=(-0.025, -0.025, -0.025),
    checker_upper=(0.025, 0.025, 0.025),
    ignore_base_ori=True,
    # debug_vis=True,
)

stack_on_pink_detector = RelativeBboxDetector(
    base_obj_name="block_pink",
    relative_pos=(0.0, 0.0, 0.05),
    relative_quat=(1.0, 0.0, 0.0, 0.0),
    checker_lower=(-0.025, -0.025, -0.025),
    checker_upper=(0.025, 0.025, 0.025),
    ignore_base_ori=True,
    # debug_vis=True,
)


@configclass
class StackBlockMetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.CALVIN
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    checker = OrOp([
        DetectedChecker(
            obj_name=obj_name,
            detector=detector,
        )
        for obj_name, detector in product(
            ["block_red", "block_blue", "block_pink"],
            [stack_on_red_detector, stack_on_blue_detector, stack_on_pink_detector],
        )
        if obj_name != detector.base_obj_name
    ])



from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass


@configclass
class OpensdorRotProngRightFork120MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e98e7fb33743442383812b68608f7006/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e98e7fb33743442383812b68608f7006_prong_right/20240826-213930_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork121MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e98e7fb33743442383812b68608f7006/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e98e7fb33743442383812b68608f7006_prong_right/20240826-220056_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork122MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e98e7fb33743442383812b68608f7006/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e98e7fb33743442383812b68608f7006_prong_right/20240826-220312_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork123MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e98e7fb33743442383812b68608f7006/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e98e7fb33743442383812b68608f7006_prong_right/20240826-214224_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork124MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e98e7fb33743442383812b68608f7006/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e98e7fb33743442383812b68608f7006_prong_right/20240826-214156_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork125MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e98e7fb33743442383812b68608f7006/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e98e7fb33743442383812b68608f7006_prong_right/20240826-214846_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork900MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/030_fork_google_16k/030_fork_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/030_prong_right/20240826-220806_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork901MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/030_fork_google_16k/030_fork_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/030_prong_right/20240826-221144_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork902MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/030_fork_google_16k/030_fork_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/030_prong_right/20240826-211723_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork903MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/030_fork_google_16k/030_fork_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/030_prong_right/20240826-214058_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork904MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/030_fork_google_16k/030_fork_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/030_prong_right/20240826-222432_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotProngRightFork905MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="fork",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/030_fork_google_16k/030_fork_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/030_prong_right/20240826-211721_no_interaction/trajectory-unified_wo_traj_v2.pkl"



# ruff: noqa: F401

from .ant_metacfg import AntMetaCfg
from .base_robot_metacfg import BaseRobotMetaCfg
from .fetch_metacfg import FetchMetaCfg
from .franka_metacfg import FrankaMetaCfg
from .franka_slide_urdf_metacfg import FrankaSlideUrdfMetaCfg
from .franka_slide_urdf_sapien_metacfg import FrankaSlideUrdfSapienMetaCfg
from .franka_urdf_metacfg import FrankaUrdfMetaCfg
from .franka_urdf_sapien_metacfg import FrankaUrdfSapienMetaCfg
from .franka_with_gripper_extension_metacfg import FrankaWithGripperExtensionMetaCfg
from .google_robot_static_metacfg import GoogleRobotStaticMetaCfg
from .h1_2_without_hand_metacfg import H12WithoutHandMetaCfg
from .h1_metacfg import H1MetaCfg
from .h1_simple_hand_metacfg import H1SimpleHandMetaCfg
from .iiwa_metacfg import IiwaMetaCfg
from .sawyer_metacfg import SawyerMetaCfg
from .sawyer_mujoco_metacfg import SawyerMujocoMetaCfg
from .ur5e_2f85_metacfg import Ur5E2F85MetaCfg
from .walker_metacfg import WalkerMetaCfg



## TODO:
# 1. Without table (on the ground), the replay success rate is ~40%
# 2. With table, the replay success rate is ~25%
# Highly suspect it's because table and ground have different collision properties

import math

from metasim.cfg.checkers import AndOp, DetectedChecker, Relative2DSphereDetector, RelativeBboxDetector
from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass

from ..base_task_metacfg import BaseTaskMetaCfg


@configclass
class SquareD0MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.ROBOSUITE
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 200
    can_tabletop = True
    traj_filepath = "data_isaaclab/source_data/robosuite/square_d0/square_d0_binary_action_zero_height_v2.pkl"
    objects = [
        PrimitiveCubeMetaCfg(
            name="peg1",
            size=[0.03, 0.03, 0.20],
            mass=0.02,
            physics=PhysicStateType.GEOM,
            color=[0.5, 0.5, 0.5],
        ),
        RigidObjMetaCfg(
            name="peg2",
            filepath="data_isaaclab/assets/robosuite/square_d0/pegs_arena/peg2.usd",
            physics=PhysicStateType.XFORM,
        ),
        RigidObjMetaCfg(
            name="SquareNut",
            filepath="data_isaaclab/assets/robosuite/square_d0/square_nut/square_nut_light.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    checker = AndOp([
        DetectedChecker(
            obj_name="SquareNut",
            detector=RelativeBboxDetector(
                base_obj_name="peg1",
                relative_pos=[0.0, 0.0, 0.05],
                relative_quat=[1.0, 0.0, 0.0, 0.0],
                checker_lower=[-math.inf, -math.inf, -math.inf],
                checker_upper=[math.inf, math.inf, 0.0],
                debug_vis=True,
            ),
        ),
        DetectedChecker(
            obj_name="SquareNut",
            detector=Relative2DSphereDetector(
                base_obj_name="peg1",
                relative_pos=[0.0, 0.0, 0.05],
                aixs=[0, 1],
                radius=0.02,
            ),
        ),
    ])
